/*-
 * SPDX-License-Identifier: BSD-2-Clause
 *
 * Copyright (c) 2022 NKI/AVL, Netherlands Cancer Institute
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, this
 *    list of conditions and the following disclaimer
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#pragma once

#include <memory>

namespace cif
{

// --------------------------------------------------------------------

template<typename Allocator = std::allocator<void>>
class list
{
  public:



  protected:

	struct list_item
	{
		list_item *m_next = nullptr;
	};

	using list_item_allocator_type = typename std::allocator_traits<Alloc>::template rebind_alloc<list_item>;
	using list_item_allocator_traits = std::allocator_traits<item_allocator_type>;

	list_item_allocator_traits::pointer get_item()
	{
		list_item_allocator_type ia(get_allocator());
		return list_item_allocator_traits::allocate(ia, 1);
	}

	template<typename ...Arguments>
	list_item *create_list_item(uint16_t column_ix, Arguments... args)
	{
		auto p = this->get_item();
		list_item_allocator_type ia(get_allocator());
		list_item_allocator_traits::construct(ia, p, std::forward<Arguments>(args)...);
		return p;
	}

	void delete_list_item(list_item *iv)
	{
		list_item_allocator_type ia(get_allocator());
		list_item_allocator_traits::destroy(ia, iv);
		list_item_allocator_traits::deallocate(ia, iv, 1);
	}

	list_item *m_head = nullptr, *m_tail = nullptr;
};


} // namespace cif
